package com.atguigu.cart.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.nacos.client.naming.utils.CollectionUtils;
import com.atguigu.cart.service.CartService;
import com.atguigu.context.BaseContext;
import com.atguigu.feign.product.ProductFeignClient;
import com.atguigu.spzx.model.entity.h5.CartInfo;
import com.atguigu.spzx.model.entity.product.ProductSku;
import com.atguigu.utils.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class CartServiceImpl implements CartService {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private ProductFeignClient productFeignClient;

    /**
     * 添加购物车
     *
     * @param skuId  商品的ID值
     * @param skuNum 商品数量
     */
    @Override
    public void addToCart(Long skuId, Integer skuNum) {
        // 当前用户
        Long userId = BaseContext.getUserInfo().getId();
        // 用户id组合，得到购物车id
        String cartKey = RedisUtils.getCartKey(userId);
        Object cartInfoJson = redisTemplate.opsForHash().get(cartKey, String.valueOf(skuId));
        // 如果不为空
        CartInfo cartInfo = null;
        if (cartInfoJson != null) {
            cartInfo = JSON.parseObject(JSON.toJSONString(cartInfoJson), CartInfo.class);
            // 数量相加
            cartInfo.setSkuNum(cartInfo.getSkuNum() + skuNum);
            // 购物车状态为选中
            cartInfo.setIsChecked(1);
            cartInfo.setUpdateTime(new Date());
        } else {
            cartInfo = new CartInfo();
            // 远程调用，根据skuId获取商品
            // 购物车数据是从商品详情得到 {skuInfo}
            ProductSku productSku = productFeignClient.getBySkuId(skuId).getData();
            cartInfo.setCartPrice(productSku.getSalePrice());
            cartInfo.setSkuNum(skuNum);
            cartInfo.setSkuId(skuId);
            cartInfo.setUserId(userId);
            cartInfo.setImgUrl(productSku.getThumbImg());
            cartInfo.setSkuName(productSku.getSkuName());
            cartInfo.setIsChecked(1);
            cartInfo.setCreateTime(new Date());
            cartInfo.setUpdateTime(new Date());
        }
        // 将商品数据存储到购物车中
        redisTemplate.opsForHash().put(cartKey, String.valueOf(skuId), cartInfo);
    }

    /**
     * 查询购物车
     *
     * @return 购物车列表
     */
    @Override
    public List<CartInfo> getCartList() {
        // 获取当前登录的用户信息
        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);

        // 获取数据
        List<Object> cartInfoList = redisTemplate.opsForHash().values(cartKey);

        if (!CollectionUtils.isEmpty(cartInfoList)) {
            return cartInfoList.stream().map(cartInfoJSON -> JSON.parseObject(JSON.toJSONString(cartInfoJSON), CartInfo.class))
                    .sorted((o1, o2) -> o2.getCreateTime().compareTo(o1.getCreateTime()))
                    .collect(Collectors.toList());
        }

        return new ArrayList<>();
    }

    /**
     * 删除购物车商品
     *
     * @param skuId 商品的ID值
     */
    @Override
    public void deleteCart(Long skuId) {
        // 获取当前登录的用户数据
        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);

        // 获取缓存对象
        redisTemplate.opsForHash().delete(cartKey, String.valueOf(skuId));
    }

    /**
     * 更新购物车商品选中状态
     *
     * @param skuId     商品的ID值
     * @param isChecked 是否选中
     */
    @Override
    public void checkCart(Long skuId, Integer isChecked) {
        // 获取当前登录的用户数据
        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);

        Boolean hasKey = redisTemplate.opsForHash().hasKey(cartKey, String.valueOf(skuId));
        if (hasKey) {
            Object cartInfoString = Objects.requireNonNull(redisTemplate.opsForHash().get(cartKey, String.valueOf(skuId)));
            String cartInfoJSON = JSON.toJSONString(cartInfoString);
            CartInfo cartInfo = JSON.parseObject(cartInfoJSON, CartInfo.class);
            cartInfo.setIsChecked(isChecked);
            redisTemplate.opsForHash().put(cartKey, String.valueOf(skuId), cartInfo);
        }
    }

    /**
     * 更新购物车商品全部选中状态
     *
     * @param isChecked 是否选中
     */
    @Override
    public void allCheckCart(Integer isChecked) {
        // 获取当前登录的用户数据
        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);

        // 获取所有的购物项数据
        List<Object> objectList = redisTemplate.opsForHash().values(cartKey);
        if (!CollectionUtils.isEmpty(objectList)) {
            objectList.stream().map(cartInfoJSON -> {
                CartInfo cartInfo = JSON.parseObject(JSON.toJSONString(cartInfoJSON), CartInfo.class);
                cartInfo.setIsChecked(isChecked);
                return cartInfo;
            }).forEach(cartInfo -> redisTemplate.opsForHash().put(cartKey, String.valueOf(cartInfo.getSkuId()), cartInfo));
        }
    }

    /**
     * 清空购物车
     */
    @Override
    public void clearCart() {
        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);
        redisTemplate.delete(cartKey);
    }

    /**
     * 选中的购物车
     *
     * @return CartInfo列表
     */
    @Override
    public List<CartInfo> getAllChecked() {
        // 获取用户id
        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);

        // 根据key获取购物车所有商品
        List<Object> objectList = redisTemplate.opsForHash().values(cartKey);

        if (!CollectionUtils.isEmpty(objectList)) {
            return objectList.stream()
                    .map(object -> JSON.parseObject(JSON.toJSONString(object), CartInfo.class))
                    .filter(cartInfo -> cartInfo.getIsChecked() == 1)
                    .collect(Collectors.toList());
        }
        return new ArrayList<>();
    }

    /**
     * 清空提交过的购物车
     */
    @Override
    public void deleteChecked() {
        log.info("清空提交过的购物车");

        Long userId = BaseContext.getUserInfo().getId();
        String cartKey = RedisUtils.getCartKey(userId);
        List<Object> objectList = redisTemplate.opsForHash().values(cartKey);       // 删除选中的购物项数据
        if (!CollectionUtils.isEmpty(objectList)) {
            objectList.stream().map(cartInfoJSON -> JSON.parseObject(JSON.toJSONString(cartInfoJSON), CartInfo.class))
                    .filter(cartInfo -> cartInfo.getIsChecked() == 1)
                    .forEach(cartInfo -> redisTemplate.opsForHash().delete(cartKey, String.valueOf(cartInfo.getSkuId())));
        }
    }
}
